MODULE_NAME='ModuleComponent'(dev vdvDev[], dev dvTP)
(***********************************************************)
(*  FILE CREATED ON: 12/09/2005  AT: 10:19:02              *)
(***********************************************************)
(*  FILE_LAST_MODIFIED_ON: 08/23/2006  AT: 14:33:38        *)
(***********************************************************)
(* Version 1.0.0 - Initial release                         *)
(* Version 1.0.1 - Made buttons internal                   *)
(***********************************************************)

DEFINE_DEVICE

DEFINE_CONSTANT

// Buttons not defined by the G4API
integer BTN_MODULE_ONLINE = 1001
integer BTN_MODULE_INITIALIZED = 1002
integer BTN_MODULE_REINIT = 1003
integer BTN_MODULE_DEBUG1 = 1004
integer BTN_MODULE_DEBUG2 = 1005
integer BTN_MODULE_DEBUG3 = 1006
integer BTN_MODULE_DEBUG4 = 1007
integer BTN_MODULE_QUERY_DEBUG = 1540
integer BTN_MODULE_QUERY_FW = 1541
integer BTN_MODULE_QUERY_VERSION = 1542
integer BTN_MODULE_SET_MONTH = 1543
integer BTN_MODULE_SET_DAY = 1544
integer BTN_MODULE_SET_YEAR = 1545
integer BTN_MODULE_SET_HOUR = 1546
integer BTN_MODULE_SET_MINUTE = 1547
integer BTN_MODULE_SET_SECONDS = 1548
integer BTN_MODULE_SET_DATE_TIME = 1549

// Variable text buttons not defined by the G4API
integer TXT_MODULE_FW = 1001
integer TXT_MODULE_VERSION = 1002
integer TXT_MODULE_MONTH = 1543
integer TXT_MODULE_DAY = 1544
integer TXT_MODULE_YEAR = 1545
integer TXT_MODULE_HOUR = 1546
integer TXT_MODULE_MINUTE = 1547
integer TXT_MODULE_SECOND = 1548

// Output buttons not defined by the G4API
integer BTN_MODULE_OUTPUT[] =	//*****************************************************************************
{				//
    861, 862, 863, 864, 865,	// This array may be adjusted to reflect how many outputs the actual device has
    866, 867, 868, 869, 870,	// If the device has more than 20 outputs, more buttons may be added, however,
    871, 872, 873, 874, 875,	// you must ensure that you do not use buttons already defined.
    876, 877, 878, 879, 880	//
}				//*****************************************************************************

#include 'SNAPI.axi'
#include 'G4API.axi'

DEFINE_VARIABLE

volatile integer nCurrentOutput = 1	// The output (zone) currently being controlled (= device / channel number!?)
volatile integer nOutputCount = 1	// The number of outputs (zones) this device has
volatile integer nDbg = 1		// The debug level

// The following variables must have a length equal to the number of outputs ~ the size of the BTN_MODULE_OUTPUT array
volatile integer nSet[20] = 0		// Tells what is being set (Month, Day, Year, Hour, Minute, Second) per output
volatile char sMonth[20][2]		// Holds the month per output
volatile char sDay[20][2]		// Holds the day per output
volatile char sYear[20][4]		// Holds the year per output
volatile char sHour[20][2]		// Holds the hour per output
volatile char sMinute[20][2]		// Holds the minute per output
volatile char sSecond[20][2]		// Holds the second per output


//*******************************************************************
// Function : updateFeedback                                        *
// Purpose  : to update fb to the touch panel when it comes online  *
// Params   : none                                                  *
// Return   : none                                                  *
//*******************************************************************
define_function updateFeedback()
{
    stack_var integer x
    
    for (x = 1; x <= nOutputCount; x++)
    {
	[dvTP, BTN_MODULE_OUTPUT[x]] = (nCurrentOutput == x)
    }
    send_command dvTP, "'^TXT-',itoa(TXT_MODULE_MONTH),',0,',sMonth[nCurrentOutput]"
    send_command dvTP, "'^TXT-',itoa(TXT_MODULE_DAY),',0,',sDay[nCurrentOutput]"
    send_command dvTP, "'^TXT-',itoa(TXT_MODULE_YEAR),',0,',sYear[nCurrentOutput]"
    send_command dvTP, "'^TXT-',itoa(TXT_MODULE_HOUR),',0,',sHour[nCurrentOutput]"
    send_command dvTP, "'^TXT-',itoa(TXT_MODULE_MINUTE),',0,',sMinute[nCurrentOutput]"
    send_command dvTP, "'^TXT-',itoa(TXT_MODULE_SECOND),',0,',sSecond[nCurrentOutput]"
    send_command dvTP, "'^TXT-',itoa(TXT_MODULE_FW),',0, '"
    send_command dvTP, "'^TXT-',itoa(TXT_MODULE_VERSION),',0, '"
}

//*******************************************************************
// Function : println						    *
// Purpose  : to print a line to the telnet session		    *
// Params   : sTxt - the data to print to the telnet session	    *
// Return   : none						    *
//*******************************************************************
define_function println (char sTxt[])
{
    if (nDbg > 2)
    {
	send_string 0, "'** Message from ModuleComponent.axs: ',sTxt"
    }
}

//*******************************************************************
// Function : dpstoa (= device port system to ascii)	            *
// Purpose  : to convert a device's DPS to ascii for displaying	    *
// Params   : dvIn - the device to be represented in ascii	    *
// Return   : char[20] - the ascii representation of the dvIn's DPS *
//*******************************************************************
define_function char[20] dpstoa (dev dvIn)
{
    return "itoa(dvIn.number),':',itoa(dvIn.port),':',itoa(dvIn.system)"
}

DEFINE_START

nOutputCount = length_array(BTN_MODULE_OUTPUT)

DEFINE_EVENT

data_event[dvTP]
{
    online:
    {
	updateFeedback()
    }
}

data_event[dvTP.number:1:0]
{
    string:
    {
	if (nSet[nCurrentOutput])
	{
	    println("'Data from TP: ',data.text")
	    if (find_string(data.text, 'ABORT', 1))
	    {
		nSet[nCurrentOutput] = 0
	    }
	    else
	    {
		remove_string(data.text, 'KEYP-', 1)
		switch (nSet[nCurrentOutput])
		{
		    case BTN_MODULE_SET_MONTH :
		    {
			if (atoi(data.text) > 12)
			{
			    send_command dvTP, "'@AKP-',data.text,';Month must be 1-12'"
			}
			else
			{
			    if (atoi(data.text) < 10)
			    {
				sMonth[nCurrentOutput] = "'0', data.text"
			    }
			    else
			    {
				sMonth[nCurrentOutput] = data.text
			    }
			    send_command dvTP, "'^TXT-',itoa(TXT_MODULE_MONTH),',0,',sMonth[nCurrentOutput]"
			    nSet[nCurrentOutput] = 0
			}
		    }
		    case BTN_MODULE_SET_DAY :
		    {
			if (atoi(data.text) > 31)
			{
			    send_command dvTP, "'@AKP-',data.text,';Day must be 1-31'"
			}
			else
			{
			    if (atoi(data.text) < 10)
			    {
				sDay[nCurrentOutput] = "'0', data.text"
			    }
			    else
			    {
				sDay[nCurrentOutput] = data.text
			    }
			    send_command dvTP, "'^TXT-',itoa(TXT_MODULE_DAY),',0,',sDay[nCurrentOutput]"
			    nSet[nCurrentOutput] = 0
			}
		    }
		    case BTN_MODULE_SET_YEAR :
		    {
			if (length_string(data.text) != 4)
			{
			    send_command dvTP, "'@AKP-',data.text,';Year must be four digits'"
			}
			else
			{
			    sYear[nCurrentOutput] = data.text
			    send_command dvTP, "'^TXT-',itoa(TXT_MODULE_YEAR),',0,',sYear[nCurrentOutput]"
			    nSet[nCurrentOutput] = 0
			}
		    }
		    case BTN_MODULE_SET_HOUR :
		    {
			if (atoi(data.text) > 24)
			{
			    send_command dvTP, "'@AKP-',data.text,';Hour must be 1-24'"
			}
			else
			{
			    if (atoi(data.text) < 10)
			    {
				sHour[nCurrentOutput] = "'0', data.text"
			    }
			    else
			    {
				sHour[nCurrentOutput] = data.text
			    }
			    send_command dvTP, "'^TXT-',itoa(TXT_MODULE_HOUR),',0,',sHour[nCurrentOutput]"
			    nSet[nCurrentOutput] = 0
			}
		    }
		    case BTN_MODULE_SET_MINUTE :
		    {
			if (atoi(data.text) > 59)
			{
			    send_command dvTP, "'@AKP-',data.text,';Minute must be 1-59'"
			}
			else
			{
			    if (atoi(data.text) < 10)
			    {
				sMinute[nCurrentOutput] = "'0', data.text"
			    }
			    else
			    {
				sMinute[nCurrentOutput] = data.text
			    }
			    send_command dvTP, "'^TXT-',itoa(TXT_MODULE_MINUTE),',0,',sMinute[nCurrentOutput]"
			    nSet[nCurrentOutput] = 0
			}
		    }
		    case BTN_MODULE_SET_SECONDS :
		    {
			if (atoi(data.text) > 59)
			{
			    send_command dvTP, "'@AKP-',data.text,';Seconds must be 1-59'"
			}
			else
			{
			    if (atoi(data.text) < 10)
			    {
				sSecond[nCurrentOutput] = "'0', data.text"
			    }
			    else
			    {
				sSecond[nCurrentOutput] = data.text
			    }
			    send_command dvTP, "'^TXT-',itoa(TXT_MODULE_SECOND),',0,',sSecond[nCurrentOutput]"
			    nSet[nCurrentOutput] = 0
			}
		    }
		}
	    }
	}
    }
}

// events from java comm module to touch panel
data_event[vdvDev]
{
    command:
    {
	if (find_string(data.text, '-', 1))
	{
	    stack_var integer x
	    stack_var integer nTempDev
	    stack_var char sCmd[20]
	    sCmd = remove_string(data.text, '-', 1)
	    
	    // trying to find out which device send the command
	    for (x = 1; x <= nOutputCount; x++)
	    {
		if (data.device == vdvDev[x])
		{
		    nTempDev = x
		    break
		}
	    }
	    
	    switch (sCmd)
	    {
		case 'FWVERSION-' :
		{
		    println("'UI received from comm on device ',itoa(nTempDev),': ',sCmd,data.text")
		    send_command dvTP, "'^TXT-',itoa(TXT_MODULE_FW),',0,',data.text"
		}
		case 'VERSION-' :
		{
		    println("'UI received from comm on device ',itoa(nTempDev),': ',sCmd,data.text")
		    // writing module version into text field on touch panel
		    send_command dvTP, "'^TXT-',itoa(TXT_MODULE_VERSION),',0,',data.text"
		}
		case 'DEBUG-' :
		{
		    println("'UI received from comm on device ',itoa(nTempDev),': ',sCmd,data.text")
		    nDbg = atoi(data.text)
		}
		case 'PROPERTY-' :
		{
		    println("'UI received from comm on device ',itoa(nTempDev),': ',sCmd,data.text")
		}
	    }
	}
    }
}

button_event[dvTP, BTN_MODULE_OUTPUT]
{
    push:
    {
	nCurrentOutput = get_last(BTN_MODULE_OUTPUT)
	updateFeedback()
    }
}

button_event[dvTP, BTN_MODULE_REINIT]
{
    push:
    {
	send_command vdvDev[nCurrentOutput], 'REINIT'
	println("'send_command ',dpstoa(vdvDev[nCurrentOutput]),', ',39,'REINIT',39")
    }
}

// set log level?
button_event[dvTP, BTN_MODULE_DEBUG1]	// ERROR
button_event[dvTP, BTN_MODULE_DEBUG2]	// WARNING
button_event[dvTP, BTN_MODULE_DEBUG3]	// INFO
button_event[dvTP, BTN_MODULE_DEBUG4]	// DEBUG
{
    push:
    {
	send_command vdvDev[nCurrentOutput], "'DEBUG-',itoa(button.input.channel - BTN_MODULE_DEBUG1 + 1)"
	println("'send_command ',dpstoa(vdvDev[nCurrentOutput]),', ',39,'DEBUG-',itoa(button.input.channel - BTN_MODULE_DEBUG1 + 1),39")
    }
}

button_event[dvTP, BTN_MODULE_QUERY_DEBUG]
{
    push:
    {
	send_command vdvDev[nCurrentOutput], "'?DEBUG'"
	println("'send_command ',dpstoa(vdvDev[nCurrentOutput]),', ',39,'?DEBUG',39")
    }
}
 
// ask device for firmware version - not in use?
button_event[dvTP, BTN_MODULE_QUERY_FW]
{
    push:
    {
	send_command vdvDev[nCurrentOutput], "'?FWVERSION'"
	println("'send_command ',dpstoa(vdvDev[nCurrentOutput]),', ',39,'?FWVERSION',39")
    }
}

// ask device for version
button_event[dvTP, BTN_MODULE_QUERY_VERSION]
{
    push:
    {
	send_command vdvDev[nCurrentOutput], "'?VERSION'"
	println("'send_command ',dpstoa(vdvDev[nCurrentOutput]),', ',39,'?VERSION',39")
    }
}

button_event[dvTP, BTN_MODULE_SET_MONTH]
button_event[dvTP, BTN_MODULE_SET_DAY]
button_event[dvTP, BTN_MODULE_SET_YEAR]
button_event[dvTP, BTN_MODULE_SET_HOUR]
button_event[dvTP, BTN_MODULE_SET_MINUTE]
button_event[dvTP, BTN_MODULE_SET_SECONDS]
{
    push:
    {
	nSet[nCurrentOutput] = button.input.channel
	switch (nSet[nCurrentOutput])
	{
	    case BTN_MODULE_SET_MONTH :
	    {
		send_command dvTP, "'@AKP-;Enter Month (##)'"
	    }
	    case BTN_MODULE_SET_DAY :
	    {
		send_command dvTP, "'@AKP-;Enter Day (##)'"
	    }
	    case BTN_MODULE_SET_YEAR :
	    {
		send_command dvTP, "'@AKP-;Enter Year (####)'"
	    }
	    case BTN_MODULE_SET_HOUR :
	    {
		send_command dvTP, "'@AKP-;Enter Hour (##)'"
	    }
	    case BTN_MODULE_SET_MINUTE :
	    {
		send_command dvTP, "'@AKP-;Enter Minute (##)'"
	    }
	    case BTN_MODULE_SET_SECONDS :
	    {
		send_command dvTP, "'@AKP-;Enter Second (##)'"
	    }
	}
    }
}

button_event[dvTP, BTN_MODULE_SET_DATE_TIME]
{
    push:
    {
	send_command vdvDev[nCurrentOutput], "'CLOCK-',sMonth[nCurrentOutput],'/',sDay[nCurrentOutput],'/',
	    sYear[nCurrentOutput],' ',sHour[nCurrentOutput],':',sMinute[nCurrentOutput],':',sSecond[nCurrentOutput]"
	println("'send_command ',dpstoa(vdvDev[nCurrentOutput]),', ',39,'CLOCK-',sMonth[nCurrentOutput],'/',sDay[nCurrentOutput],'/',
	    sYear[nCurrentOutput],' ',sHour[nCurrentOutput],':',sMinute[nCurrentOutput],':',sSecond[nCurrentOutput],39")
    }
}

DEFINE_PROGRAM
[dvTP, BTN_MODULE_ONLINE] = [vdvDev[nCurrentOutput], DEVICE_COMMUNICATING]
[dvTP, BTN_MODULE_INITIALIZED] = [vdvDev[nCurrentOutput], DATA_INITIALIZED]
[dvTP, BTN_MODULE_DEBUG1] = (nDbg == 1)
[dvTP, BTN_MODULE_DEBUG2] = (nDbg == 2)
[dvTP, BTN_MODULE_DEBUG3] = (nDbg == 3)
[dvTP, BTN_MODULE_DEBUG4] = (nDbg == 4)
